@@ -1,5 +1,5 @@
 /*
- * "$Id: print-pcl.c,v 1.47.4.6 2002/12/08 21:12:56 rlk Exp $"
+ * "$Id: print-pcl.c,v 1.47.4.13 2004/06/09 10:29:58 davehill Exp $"
  *
  *   Print plug-in HP PCL driver for the GIMP.
  *
@@ -441,9 +441,35 @@ static const pcl_cap_t pcl_model_capabilities[] =
       -1,
     },
   },
-  /* DesignJet 2500C/3500C (44" wide) */
+  /* DesignJet 2000C/2500C (36" wide) */
   { 12500,	/* Deskjet 2500 already has "2500" */
-    44 * 72, 150 * 12 * 72,		/* 150ft in roll mode, 64" in sheet */
+    36 * 72, 150 * 12 * 72,		/* 150ft in roll mode, 64" in sheet */
+    830 * 72 / 100, 583 * 72 / 100,	/* 8.3" wide min in sheet mode */
+    PCL_RES_300_300 | PCL_RES_600_600_MONO,
+    {49, 49, 15, 15},		/* Check/Fix */
+    {49, 49, 15, 15},
+    PCL_COLOR_CMYK,
+    PCL_PRINTER_DJ | PCL_PRINTER_TIFF | PCL_PRINTER_BLANKLINE | PCL_PRINTER_CUSTOM_SIZE | PCL_PRINTER_NEW_ERG,
+    {
+      PCL_PAPERSIZE_LETTER,
+      -1,
+    },
+    {
+      PCL_PAPERTYPE_PLAIN,
+      PCL_PAPERTYPE_BOND,
+      PCL_PAPERTYPE_PREMIUM,
+      PCL_PAPERTYPE_GLOSSY,
+      PCL_PAPERTYPE_TRANS,
+      -1,
+    },
+    {
+      PCL_PAPERSOURCE_STANDARD,
+      -1,
+    },
+  },
+  /* DesignJet 3000C/3500C (54" wide) */
+  { 13500,	/* Deskjet 2500 already has "2500" */
+    54 * 72, 150 * 12 * 72,		/* 150ft in roll mode, 64" in sheet */
     830 * 72 / 100, 583 * 72 / 100,	/* 8.3" wide min in sheet mode */
     PCL_RES_300_300 | PCL_RES_600_600_MONO,
     {49, 49, 15, 15},		/* Check/Fix */
@@ -472,8 +498,8 @@ static const pcl_cap_t pcl_model_capabilities[] =
     17 * 72 / 2, 14 * 72,
     1, 1,				/* Min paper size */
     PCL_RES_150_150 | PCL_RES_300_300,
-    {7, 41, 18, 18},
-    {7, 41, 10, 10},	/* Check/Fix */
+    {6, 48, 18, 18},	/* from bpd07933.pdf */
+    {6, 48, 10, 11},	/* from bpd07933.pdf */
     PCL_COLOR_CMY,
     PCL_PRINTER_DJ | PCL_PRINTER_TIFF | PCL_PRINTER_BLANKLINE,
     {
@@ -1193,6 +1219,40 @@ static const pcl_cap_t pcl_model_capabilities[] =
       -1,
     },
   },
+  /* LaserJet IIP (TIFF but no blankline) */
+  { 21,
+    17 * 72 / 2, 14 * 72,
+    1, 1,				/* Min paper size */
+    PCL_RES_150_150 | PCL_RES_300_300,
+    {12, 12, 18, 18},
+    {12, 12, 10, 10},	/* Check/Fix */
+    PCL_COLOR_NONE,
+    PCL_PRINTER_LJ | PCL_PRINTER_TIFF,
+    {
+      PCL_PAPERSIZE_EXECUTIVE,
+      PCL_PAPERSIZE_STATEMENT,
+      PCL_PAPERSIZE_LETTER,
+      PCL_PAPERSIZE_LEGAL,
+      PCL_PAPERSIZE_A4,
+      PCL_PAPERSIZE_MONARCH_ENV,
+      PCL_PAPERSIZE_COMMERCIAL10_ENV,
+      PCL_PAPERSIZE_DL_ENV,
+      PCL_PAPERSIZE_C5_ENV,
+      PCL_PAPERSIZE_C6_ENV,
+      -1,
+    },
+    { -1,			/* No selectable paper types */
+    },
+    {
+      PCL_PAPERSOURCE_STANDARD,
+      PCL_PAPERSOURCE_MANUAL,
+      PCL_PAPERSOURCE_LJ_TRAY1,
+      PCL_PAPERSOURCE_LJ_TRAY2,
+      PCL_PAPERSOURCE_LJ_TRAY3,
+      PCL_PAPERSOURCE_LJ_TRAY4,
+      -1,
+    },
+  },
   /* LaserJet III series */
   { 3,
     17 * 72 / 2, 14 * 72,
@@ -2674,8 +2734,8 @@ pcl_print(const stp_printer_t printer,		/* I - Model */
     }
   stp_dither_set_density(dither, stp_get_density(nv));
 
-  in  = stp_malloc(image_width * image_bpp);
-  out = stp_malloc(image_width * out_bpp * 2);
+  in  = stp_zalloc(image_width * image_bpp);
+  out = stp_zalloc(image_width * out_bpp * 2);
 
   errdiv  = image_height / out_height;
   errmod  = image_height % out_height;
@@ -2692,12 +2752,18 @@ pcl_print(const stp_printer_t printer,		/* I - Model */
 #endif
 
   dt = stp_create_dither_data();
-  stp_add_channel(dt, black, ECOLOR_K, 0);
-  stp_add_channel(dt, cyan, ECOLOR_C, 0);
-  stp_add_channel(dt, lcyan, ECOLOR_C, 1);
-  stp_add_channel(dt, magenta, ECOLOR_M, 0);
-  stp_add_channel(dt, lmagenta, ECOLOR_M, 1);
-  stp_add_channel(dt, yellow, ECOLOR_Y, 0);
+  if (black)
+    stp_add_channel(dt, black, ECOLOR_K, 0);
+  if (cyan)
+    stp_add_channel(dt, cyan, ECOLOR_C, 0);
+  if (lcyan)
+    stp_add_channel(dt, lcyan, ECOLOR_C, 1);
+  if (magenta)
+    stp_add_channel(dt, magenta, ECOLOR_M, 0);
+  if (lmagenta)
+    stp_add_channel(dt, lmagenta, ECOLOR_M, 1);
+  if (yellow)
+    stp_add_channel(dt, yellow, ECOLOR_Y, 0);
 
   for (y = 0; y < out_height; y ++)
   {
